"use strict";

require("source-map-support/register");

var _http = require("http");

var _http2 = _interopRequireDefault(_http);

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _jsonfile = require("jsonfile");

var _jsonfile2 = _interopRequireDefault(_jsonfile);

var _recursiveReaddir = require("recursive-readdir");

var _recursiveReaddir2 = _interopRequireDefault(_recursiveReaddir);

var _ramda = require("ramda");

var _ramda2 = _interopRequireDefault(_ramda);

var _url = require("url");

var _url2 = _interopRequireDefault(_url);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var dotfilePath = require("os").homedir() + "/.jaskServerConfig.js";
var config = {};

var getAllEventsIdents = function getAllEventsIdents() {
	return new Promise(function (done) {
		(0, _recursiveReaddir2.default)(config.dataFolder, function (err, files) {
			done(files || []);
		});
	}).then(function (files) {
		return Promise.all(_ramda2.default.pipe(_ramda2.default.sortBy(_ramda2.default.pipe(_ramda2.default.replace(/.+\//, ""), function (x) {
			return parseInt(x, 10);
		})), _ramda2.default.map(_ramda2.default.replace(config.dataFolder + "/", "")))(files));
	}).then(function (x) {
		return JSON.stringify(x);
	});
};

var getEvent = function getEvent(path) {
	return new Promise(function (done) {
		return _jsonfile2.default.readFile(config.dataFolder + path, function (err, dat) {
			return done(dat);
		});
	}).then(function (x) {
		return JSON.stringify(x);
	});
};

var startServer = function startServer() {
	_http2.default.createServer(function (req, res) {
		var parsedUrl = _url2.default.parse(req.url);

		if (req.headers.authorization !== config.key) {
			res.statusCode = 401;
			return res.end('{err:401, msg: "not authorised, please provide key"}');
		}

		if (parsedUrl.pathname === "/") {
			res.setHeader("Content-type", "application/json");
			getAllEventsIdents().then(function (data) {
				return res.end(data);
			});
		} else {
			res.setHeader("Content-type", "application/json");
			getEvent(parsedUrl.pathname).then(function (data) {
				return res.end(data);
			});
		}
	}).listen(config.port);

	console.log("\nServer started\n==============\n\t+ on port " + config.port + "\n\t+ serving from " + config.dataFolder + "\n\t+ with auth key: \"" + config.key + "\"\n");
};

console.log("looking for config file at " + dotfilePath);
try {
	config = require(dotfilePath);
	startServer();
} catch (e) {
	if (e.code === "MODULE_NOT_FOUND") {
		var possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
		var generateKey = function generateKey() {
			return _ramda2.default.times(function () {
				return possible.charAt(Math.floor(Math.random() * possible.length));
			}, 128).join("");
		};

		_fs2.default.writeFile(dotfilePath, "module.exports = {\n\tdataFolder: `" + require("os").homedir() + "/.jaskActions`,\n\tport: 9000,\n\tkey: \"" + generateKey() + "\",\n}", function () {
			console.log("created a config file at " + dotfilePath);
			config = require(dotfilePath);
			startServer();
		});
	}
}